
#ifndef UtMatrizH
#define UtMatrizH

#include <fstream>
using namespace std;
//---------------------------------------------------------------------------

template <class T> class Matriz
{
public:
        Matriz() {matriz = NULL; nCols = nFils = 0;};
        Matriz(unsigned int c, unsigned int f) {matriz = NULL; init(c, f);};
        Matriz(unsigned int c, unsigned int f, const T& e) {matriz = NULL; init(c, f, e);};
        ~Matriz();
        unsigned int getCols() const {return nCols;};
        unsigned int getFils() const {return nFils;};
        bool getElem(unsigned int c, unsigned int f, T& e) const;
        bool setElem(unsigned int c, unsigned int f, const T& e);
        bool operator==(const Matriz<T>& mat) const;
        bool operator!=(const Matriz<T>& mat) const {return!(*this == mat);};
        Matriz* operator=(const Matriz<T>& mat);
        bool salvar(ofstream& sal) const;
        bool cargar(ifstream& ent);

private:
        unsigned int nCols, nFils;
        T** matriz;
        bool inRange(unsigned int c, unsigned int f) const;
        void init(unsigned int c, unsigned int f);
        void init(unsigned int c, unsigned int f, const T& e);
};
//---------------------------------------------------------------------------

template <class T>
void Matriz<T>::init(unsigned int c, unsigned int f)
{
        if(matriz != NULL)
        {
                for(unsigned int i = 0; i < nCols; i++)
                {
                        delete[] matriz[i];
                }
                delete[] matriz;
        }
        nCols = c;
        nFils = f;
        matriz = NULL;
        if((c > 0) && (f > 0))
        {
                matriz = new T*[nCols];
                for(unsigned int i = 0; i < nCols; i++)
                {
                        matriz[i] = new T[nFils];
                }
        }
}

template <class T>
void Matriz<T>::init(unsigned int c, unsigned int f, const T& e)
{
        init(c, f);
        if(matriz != NULL)
        {
                for(unsigned int i = 0; i < c; i++)
                {
                        for(unsigned int j = 0; j < f; j++)
                        {
                                matriz[i][j] = e;
                        }
                }
        }
}

template <class T>
Matriz<T>::~Matriz()
{
        for(unsigned int i = 0; i < nCols; i++)
        {
                delete[] matriz[i];
        }
        delete[] matriz;
        matriz = NULL;
}

template <class T>
bool Matriz<T>::getElem(unsigned int c, unsigned int f, T& e) const
{
        if(inRange(c, f))
        {
                e = matriz[c][f];
                return true;
        }
        else return false;
}

template <class T>
bool Matriz<T>::setElem(unsigned int c, unsigned int f, const T& e)
{
        if(inRange(c, f))
        {
                matriz[c][f] = e;
                return true;
        }
        else return false;
}

template <class T>
bool Matriz<T>::inRange(unsigned int c, unsigned int f) const
{
        return ((c < nCols) && (f < nFils));
}

template <class T>
bool Matriz<T>::operator==(const Matriz<T>& mat) const
{
        bool iguales = true;
        int i = 0;
        int j = 0;
        T aux;
        if((mat->getCols != nCols) || (mat->getFils != nFils))
        {
                return false;
        }

        while((iguales) && (i < nCols))
        {
                while((iguales) && (j < nFils))
                {
                        if(mat->getElem(i, j, aux)
                        {
                                if(matriz[i][j] != aux)
                                {
                                        iguales = false;
                                }
                        }
                        else
                        {
                                iguales = false;
                        }
                        ++j;
                }
                j = 0;
                ++i;
        }

        ifstream sal;

        return iguales;
}

template <class T>
bool Matriz<T>::salvar(ofstream& sal) const
{
        sal.write((char*)& nCols, sizeof(unsigned int));
        sal.write((char*)& nFils, sizeof(unsigned int));
        if(matriz != NULL)
        {
                for(int i = 0; i < nCols; i++)
                {
                        for(int j = 0; j < nFils; j++)
                        {
                                sal.write((char*)& matriz[i][j], sizeof(T));
                        }
                }
        }
        return sal;
}

template <class T>
bool Matriz<T>::cargar(ifstream& ent)
{
        unsigned int c, f;
        ent.read((char*)& c, sizeof(unsigned int));
        ent.read((char*)& f, sizeof(unsigned int));
        init(c, f);
        if((nCols > 0) && (nFils > 0))
        {
                for(int i = 0; i < nCols; i++)
                {
                        for(int j = 0; j < nFils; j++)
                        {
                                ent.read((char*)& matriz[i][j], sizeof(T));
                        }
                }
        }
        return ent;
}

template <class T>
Matriz<T>* Matriz<T>::operator=(const Matriz<T>& mat)
{
        init(mat.getCols(), mat.getFils());
        for(unsigned int i = 0; i < nCols; i++)
        {
                for(unsigned int j = 0; j < nFils; j++)
                {
                        matriz[i][j] = mat.matriz[i][j];
                }
        }
        return this;
}

#endif
